var url = require("url");
var path = require("path"); 
var fs = require("fs");
var hlp = require('../helper');

function handleHttpStream(_req, _res, _next, _bridge, _logger, _config) {
	
	function reportError(err) {
		_logger.log('error', err);
		_res.writeHead(500);
		_res.end('Internal Server Error');
	}

	if (_req.method != 'GET' || _req.url == "/") {
        _next();
        return;
    }

 	var parts = url.parse(_req.url);
    var filePath = unescape(path.normalize('.' + parts.pathname));
    var urlFile = filePath;
    filePath = path.normalize(_config.webRootFolder + "/" + filePath);
    var targetFileName = path.basename(filePath);

	var clientIPAddress = hlp.formatIP(_req.connection.remoteAddress);

	var alias = "";
	var isMediaFile	= false;
	var validHTTPStream = false;
    var isStreamPath;
    var originalFile = filePath;
    var isTempFile = false;
	var userAgent = _req.headers["user-agent"];
	var trgFile = _bridge.getTargetFile(filePath, targetFileName, urlFile, 
		clientIPAddress, userAgent);
	_logger.log('silly', 'trgFile: ', trgFile);

	if (trgFile._isError && trgFile._isAliasUnavailable) {
		return reportError('Alias \'' + urlFile + '\' is unavailable.');
	} else if (trgFile._filePath != "") {
		originalFile = trgFile._origFile;
		filePath = trgFile._filePath;
		isTempFile = trgFile._isTempFile;
		validHTTPStream = trgFile._validHttpStream;
	}
	isStreamPath = trgFile._isStreamPath;
	if (trgFile._alias != "") {
		alias = trgFile._alias;
	} 

	fs.exists(filePath, (exists) => {
		if (exists) {
			if (isStreamPath && _config.hasGroupNameAliases && 
					!validHTTPStream) { //attempt to access the playlist via 
										//direct folder access
				return reportError('File \'' + filePath + '\' not found.');
			} else if (_bridge.isStreamingFile(filePath)) {
				var sfr = _bridge.streamFile(isTempFile ? originalFile : 
					filePath, clientIPAddress, (alias == "") ? urlFile : alias, 
					(alias != ""), targetFileName, urlFile, isTempFile);
				if (sfr._errorCode == 0) {
					isMediaFile = sfr._isMediaFile;
				} else if (sfr._errorCode == 1) {
					return reportError('Alias \'' + urlFile + '\' not found.');
				} else if (sfr._errorCode == 2) {
					return reportError(
						'\'' + filePath + '\' is not a valid media file.');
				} else if (sfr._errorCode == 3) {
					return reportError(
						'Group name aliasing is enabled; ' + 
						'cannot directly playback \'' + filePath + '\'.');
				}

            }
            else if (_bridge.isMediaFile(filePath)) {
                isMediaFile = true;
            }

            fs.stat(filePath, (err, stat) => {
				var rs;
				if (err) {
					return reportError(err);
				}

				var ext = path.extname(filePath);
				if (ext != "") {
					ext = ext.substring(1);		
				} 

				var mimeType = _bridge.getMimeType(ext);
				var rangeRequest = readRangeHeader(_req.headers['range'], 
						stat.size);
				if (rangeRequest == null) {					
					var msg = '[' + 
					hlp.formatIP(_req.connection.remoteAddress) + ':' + 
						_req.connection.remotePort + '] Serving ' + 
						originalFile;
					_logger.log('info', msg);

					_res.setHeader('Content-Length', stat.size);
					_res.setHeader('Content-Type', mimeType);
					_res.setHeader('Accept-Ranges', 'bytes');
					_res.writeHead(200);
					var startTime = new Date() / 1000;
					rs = fs.createReadStream(filePath);
					rs.on('error', reportError);
					rs.on('end', () => {
						_logger.log('debug', "File served:", filePath);
						if (isTempFile) { //do not cache
							_bridge.addTempFile(filePath);
			            }
			            if (isMediaFile && sfr._errorCode != 4) {
			        		var endTime = new Date() / 1000;
			                _bridge.updateLastAccess(clientIPAddress, filePath, 
			                	startTime, endTime, false, urlFile);
			            }
					});
					//_next();
					//return;
				} else {
					var start = rangeRequest.Start;
				    var end = rangeRequest.End;

				    var msg = '[' + 
						hlp.formatIP(_req.connection.remoteAddress) + ':' + 
						_req.connection.remotePort + 
						'] Ranged Request: Serving ' + originalFile + ' {' + 
						start + '-' + end + '/' + stat.size + '} {' + 
						_req.headers['user-agent'] + '}';
					_logger.log('info', msg);
					
					// If the range can't be fulfilled. 
				    if (start >= stat.size || end >= stat.size) {
				        // Indicate the acceptable range.
				        _res.setHeader('Content-Range', 'bytes */' + 
				        	stat.size); // File size.
				        _res.writeHead(416);		
				    } else {
				    	// Indicate the current range. 
					    _res.setHeader('Content-Range', 
					    	'bytes ' + start + '-' + end + '/' + stat.size);
					    _res.setHeader('Content-Length', 
					    	start == end ? 0 : (end - start + 1));
					    _res.setHeader('Content-Type', mimeType);
					    _res.setHeader('Accept-Ranges', 'bytes');
					    _res.setHeader('Cache-Control', 'no-cache');
					    // Return the 206 'Partial Content'.
					    _res.writeHead(206);
					    var startTime = new Date() / 1000;
				    	rs = fs.createReadStream(filePath, 
				    		{start: start, end: end});
						rs.on('error', reportError);
						rs.on('readable', () => {
							_logger.log('debug', "Chunk served:", filePath);
							if (isTempFile) { //do not cache
				                _bridge.addTempFile(filePath);
				            }
				            if (isMediaFile && sfr._errorCode != 4) {
				            	var endTime = new Date() / 1000;
				                _bridge.updateLastAccess(clientIPAddress, 
				                	filePath, startTime, endTime, false, 
				                	urlFile);
				            }
						});
				    }		
				}
				rs.pipe(_res);
			});
		} else {
			_next();
		}
	});
}

function readRangeHeader(range, totalLength) {
        /*
         * Example of the method 'split' with regular expression.
         * 
         * Input: bytes=100-200
         * Output: [null, 100, 200, null]
         * 
         * Input: bytes=-200
         * Output: [null, null, 200, null]
         */

    if (range == null || range.length == 0) {
        return null;
    }

    var array = range.split(/bytes=([0-9]*)-([0-9]*)/);
    var start = parseInt(array[1]);
    var end = parseInt(array[2]);
    var result = {
        Start: isNaN(start) ? 0 : start,
        End: isNaN(end) ? (totalLength - 1) : end
    };
    
    if (!isNaN(start) && isNaN(end)) {
        result.Start = start;
        result.End = totalLength - 1;
    }

    if (isNaN(start) && !isNaN(end)) {
        result.Start = totalLength - end;
        result.End = totalLength - 1;
    }

    return result;
}

exports.handle = function(_bridge, _logger, _config) {
	return function(_req, _res, _next) {
		handleHttpStream(_req, _res, _next, _bridge, _logger, _config);
	}
};